/*
 * dvddisc: access to the content of a DVD Video disc
 * Copyright (C) 1999 Christian Wolff for convergence integrated media GmbH
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
 * 
 * The author can be reached at scarabaeus@convergence.de, 
 * the project's page is at http://linuxtv.org/dvd/
 */

#include <stdio.h>
#include <stdlib.h>
#include <dirent.h>
#include <limits.h>
#include <string.h>
#include <sys/stat.h>

#include "dvd_disc.h"
#include "dvd_file.h"
#include "dvd_udf.h"


#ifndef u8
#define u8 unsigned char
#endif

#ifndef u16
#define u16 unsigned short
#endif

#ifndef u32
#define u32 unsigned long
#endif

#ifndef u64
#define u64 unsigned long long
#endif

// are we using UDF access or our own file access?
int UseUDF=0;

// retreive the length of the IFO file in blocks - 1
int getIFOlength(u8 *data) 
{
	return (((u32)data[28]<<24)|((u32)data[29]<<16)|((u32)data[30]<<8)|(u32)data[31]);
}

// Reads IFO File or IFO Backup File 
// vtsn is 0 for VIDEO_TS.IFO, or 1 thru 99 for VTS_xx_0.IFO (where xx is vtsn)
// if backup is zero, first the IFO and then, on failure, the BUP will be read.
// if backup is not zero, only the backup file will be read (.BUP)
// *infodata will be allocated to size of file
// returns 0 or lb number of ifo file, or -1 on error
long int DiscReadIFO(int vtsn, int backup, unsigned long int ifo_start, unsigned char **infodata) 
{

	char filename[PATH_MAX];
	char filename_ifo[PATH_MAX];
	char filename_bup[PATH_MAX];
	int blocks;
	u32 filesize;
	int bup=0;  // do we use the backup?
	long int lb,start_lb=0;
	int err;
	int request;
	
//printf("-- DiscReadIFO(%d,%d,%ld);\n",vtsn,backup,ifo_start);

	if ((vtsn<0) || (vtsn>99)) return -1;  // illegal title number
	request=((vtsn)?2:1);
	
	if (vtsn) {
		sprintf(filename,DVD_VIDEO_TITLE,vtsn,0);
	} else {
		strcpy(filename,DVD_VIDEO_INDEX);
	}
	strcpy(filename_ifo,filename);
	strcat(filename_ifo,DVD_INFO_FILE);
	strcpy(filename_bup,filename);
	strcat(filename_bup,DVD_INFO_BACKUP);
	
	if (UseUDF) {  // Block Device or Image File access
		while (1) {
			if (bup) {
				sprintf(filename,"/" DVD_VIDEO_PATH "/%s",filename_bup);
			} else {
				sprintf(filename,"/" DVD_VIDEO_PATH "/%s",filename_ifo);
			}
			if ((*infodata=(u8 *)malloc(DVD_VIDEO_LB_LEN))==NULL) return -1;
			if (request==1) {  // VMG
				if ((lb=UDFFindFile(filename,&filesize))==0) {  // file not found
					if (bup++) {  // was this already the backup we tried?
						lb=0;
						err=1;
						while (err && (lb<1000)) {  // search for VMGI on the first 1000 blocks
							if (UDFReadLB(lb,1,*infodata)>0) {
								if (!strncmp(*infodata,"DVDVIDEO-VMG",12)) err=0;
								else lb++;
							} else lb++;
						}
						if (err) {  // VMGI not found
							return -1;
						} else start_lb=lb;
					}
					continue;
				} else if (!bup) start_lb=lb;  // we need the lb of the IFO, not the BUP, here!
			} else if (request==2) {  // VTS
				if (ifo_start) {
					start_lb = lb = ifo_start;
				} else {
					if ((lb=UDFFindFile(filename,&filesize))==0) {
						if (bup++) return -1;
						continue;
					} else {
						if (!bup) start_lb=lb;  // we need the lb of the IFO, not the BUP, here!
					}
				}
			}
			if (UDFReadLB(lb,1,*infodata)<=0) {
				free(*infodata);
				if (bup++) return -1;
				continue;
			}
			blocks=getIFOlength(*infodata);
			if ((*infodata=(u8 *)realloc(*infodata,(blocks+1)*DVD_VIDEO_LB_LEN))==NULL) {
				return -1;
			}
			if (UDFReadLB(lb+1,blocks,&(*infodata)[DVD_VIDEO_LB_LEN])<=0) {
				free(*infodata);
				if (bup++) return -1;
				continue;
			} else {
				return start_lb;
			}
		}
	} else {  // File System access
		if ((FileReadFile(filename_ifo,infodata))>0) return 0;
		else if ((FileReadFile(filename_bup,infodata))>0) return 0;
	}
	return -1;
}

// reads Logical Block of a vmg menu (VIDEO_TS.VOB)
// lbnum is the logical block number in the VMG
// vmg_start is the position of the VMG on the disc or 0 for file access
// data will be filled with 2048 bytes of data on success
// return 0 on success, >0 on error
int DiscReadVMGM(long int lbnum, long int vmg_start, unsigned char *data) 
{
	if (UseUDF) {
		return (UDFReadLB(vmg_start+lbnum,1,data)<=0);
	} else {
		return FileReadVMGM(lbnum,data);
	}
}

// reads Logical Block of a vts menu
// vtsn is 1 thru 99 for VTS_xx_0.VOB (where xx is vtsn)
// lbnum is the logical block number in the VMG
// vtsm_start is the position of the VTSM on the disc or 0 for file access
// data will be filled with 2048 bytes of data on success
// return 0 on success, >0 on error
int DiscReadVTSM(int vtsn, long int lbnum, long int vtsm_start, unsigned char *data) 
{
	if (UseUDF) {
		return (UDFReadLB(vtsm_start+lbnum,1,data)<=0);
	} else {
 	  return FileReadVTSM(vtsn,lbnum,data);
	}
}

// reads Logical Block of a video title set
// vtsn is 1 thru 99 for VTS_xx_y.VOB (where xx is vtsn and y is 1 thru 9)
// lbnum is the logical block number in the VMG
// vts_start is the position of the VTS on the disc or 0 for file access
// data will be filled with 2048 bytes of data on success
// return 0 on success, 1 on error
int DiscReadVTS(int vtsn, long int lbnum, long int vts_start, unsigned char *data) 
{
	if (UseUDF) {
		return (UDFReadLB(vts_start+lbnum,1,data)<=0);
	} else {
		return FileReadVTS(vtsn,lbnum,data);
	}
}


// DVD Copy Management:
// RPC - Region Playback Control
// CSS - Content Scrambling System

// Query RPC status of the drive
// type: 0=NONE (no drive region setting)
//       1=SET (drive region is set
//       2=LAST CHANCE (drive region is set, only one change remains)
//       3=PERM (region set permanently, may be reset by vendor)
// vra: number of vendor resets available
// ucca: number of user controlled changes available
// region_mask: the bit of the drive's region is set to 0, all other 7 bits to 1
// rpc_scheme: 0=unknown, 1=RPC Phase II, others reserved
// returns 0 on success, <0 on error
int DiscRPCGet(int *type, int *vra, int *ucca, int *region_mask, int *rpc_scheme) 
{
	if (UseUDF) {
		return UDFRPCGet(type,vra,ucca,region_mask,rpc_scheme);
	} else {
		return -1;
	}
}

// Set new Region for drive
// region_mask: the bit of the new drive's region is set to 0, all other 7 bits to 1
int DiscRPCSet(int region_mask) 
{
	if (UseUDF) {
		return UDFRPCSet(region_mask);
	} else {
		return -1;
	}
}

// Initializes the CSS process with the drive
// returns positive AGID on success, -1 on error, -2 if no CSS support in CD-ROM driver
int DiscCSSRequestAGID(void) 
{
	if (UseUDF) {
		return UDFCSSRequestAGID();
	} else {
		return -1;
	}
}

// Invalidates previously released AGID
// returns 0 on success, <0 on error
int DiscCSSInvalidateAGID(int agid) 
{
	if (UseUDF) {
		return UDFCSSInvalidateAGID(agid);
	} else {
		return -1;
	}
}

// Post host challenge (10 bytes) into drive
// and retreive drive response (5 bytes) for host
// returns 0 on success, -1 on error, -2 if no CSS support in CD-ROM driver
int DiscCSSDriveAuth(char *data) 
{
	if (UseUDF) {
		return UDFCSSDriveAuth(data);
	} else {
		return -1;
	}
}

// Retreive drive challenge (10 bytes)
// returns 0 on success, -1 on error, -2 if no CSS support in CD-ROM driver
int DiscCSSHostAuthChallenge(char *data) 
{
	if (UseUDF) {
		return UDFCSSHostAuthChallenge(data);
	} else {
		return -1;
	}
}

// Post host response (5 bytes) into drive
// returns 0 on success, -1 on error, -2 if no CSS support in CD-ROM driver
int DiscCSSHostAuthResponse(char *data) 
{
	if (UseUDF) {
		return UDFCSSHostAuthResponse(data);
	} else {
		return -1;
	}
}

// Retreive disc key (2048 byte) from drive
// returns 0 on success, -1 on error, -2 if no CSS support in CD-ROM driver
int DiscCSSDiscKey(char *data) 
{
	if (UseUDF) {
		return UDFCSSDiscKey(data);
	} else {
		return -1;
	}
}

// Retreive title key (5 byte) from drive
// input parameters:
//   lba: absolute number of logical block containing the title key
// return parameters:
//   data: will be filled with the bus obfuscated title key
//   cpm: 0=material not copyrighted, 1=copyrighted material
//   cp_sec: 1=this sector has specific structure for copyright protection system,
//           0=it doesn't
//   cgms: 0=copying is permitted,                                                 */ 
//         2=one generation of copies may be made,
//         3=no copying is allowed
// returns 0 on success, <0 on error
int DiscCSSTitleKey(unsigned long int lba, char *data, int *cpm, int *cp_sec, int *cgms) 
{
	if (UseUDF) {
		return UDFCSSTitleKey(lba,data,cpm,cp_sec,cgms);
	} else {
		return -1;
	}
}


// Set the path to the UDF mounted DVD disc
// Or the Path to the VIDEO_TS directory
// Or the filename of the udf block device or image file
// returns 0 on success, -1 on 'dvd not found', -2 on 'path not found'
int DiscSetVideoPath(char *Path, int LB_Length) 
{
	struct stat filestat;
	u32 filesize;

	UseUDF=((stat(Path,&filestat)<0) || (!S_ISDIR(filestat.st_mode)));
	if (UseUDF) {  // Block access to DVD
		if (UDFOpenDisc(Path)<0) return -1;
		if (UDFFindFile("/" DVD_VIDEO_PATH "/",&filesize)==0) return -2;
		return 0;
	} else {  // File access to DVD
		return FileSetVideoPath(Path,LB_Length);
	}
}
