#ifndef DVD_DISC_H
#define DVD_DISC_H

#define DVD_DISC_VERSION 20000215


// Definitions of the file names on a DVD Video and DVD Audio

// Path names
#define DVD_VIDEO_PATH     "VIDEO_TS"       // /VIDEO_TS/ directory, holding DVD Video data
#define DVD_AUDIO_PATH     "AUDIO_TS"       // /AUDIO_TS/ directory, holding DVD Audio data
#define DVD_JACKET_PATH    "JACKET_P"       // /JACKET_P/ directory, holding DVD Jacket Pictures

// Extensions
#define DVD_INFO_FILE      ".IFO"           // File extension for Info Files
#define DVD_INFO_BACKUP    ".BUP"           // File extension for Info File Backups
#define DVD_VIDEO_OBJECT   ".VOB"           // File extension for Video Objects
#define DVD_AUDIO_OBJECT   ".AOB"           // File extension for Audio Objects
#define DVD_JACKET_OBJECT  ".MP2"           // File extension for MPEG2 Jacket Pictures

// File names and file name formats
#define DVD_VIDEO_INDEX    "VIDEO_TS"       // /VIDEO_TS/VIDEO_TS.VOB or VIDEO_TS.IFO
#define DVD_VIDEO_TITLE    "VTS_%02d_%d"    // e.g. /VIDEO_TS/VTS_01_1.VOB or VTS_01_0.IFO
#define DVD_AUDIO_INDEX    "AUDIO_TS"       // /AUDIO_TS/AUDIO_TS.VOB or AUDIO_TS.IFO
#define DVD_AUDIO_SIMPLE   "AUDIO_PP"       // /AUDIO_TS/AUDIO_PP.IFO
#define DVD_AUDIO_STILL    "AUDIO_SV"       // /AUDIO_TS/AUDIO_SV.VOB or AUDIO_SV.IFO
#define DVD_AUDIO_TITLE    "ATS_%02d_%d"    // e.g. /VIDEO_TS/VTS_01_1.VOB or VTS_01_0.IFO
#define DVD_JACKET_PICTURE "J00___%c%c"     // e.g. /JACKET_P/J00___6L.MP2
#define DVD_JACKET_525     '5'              // 525 lines jacket picture
#define DVD_JACKET_625     '6'              // 625 lines jacket picture
#define DVD_JACKET_LARGE   'L'              // large jacket picture (720x480 or 720x576)
#define DVD_JACKET_MEDIUM  'M'              // medium size jacket picture (176x112 or 176x144)
#define DVD_JACKET_SMALL   'S'              // small jacket picture (96x64 or 96x80)


/***********************************************************************************/
/* Reads Video IFO File or IFO Backup File                                         */
/* vtsn is 0 for VIDEO_TS.IFO, or 1 thru 99 for VTS_xx_0.IFO (where xx is vtsn)    */
/* if backup is zero, first the IFO and then, on failure, the BUP will be read.    */
/* if backup is not zero, only the backup file will be read (.BUP)                 */
/* ifo_start is the logical block number position of the IFO data,                 */
/*   it can be 0 for file access                                                   */
/* *infodata will be allocated to the size of the file                             */
/* returns 0 on error, number of blocks read on success                            */
/***********************************************************************************/
long int DiscReadIFO(int vtsn, int backup, unsigned long int ifo_start, unsigned char **infodata);

/***********************************************************************************/
/* reads Logical Block (2048 byte) of a vmg menu (VIDEO_TS.VOB)                    */
/* lbnum is the logical block number in the VMG                                    */
/* vmg_start is the position of the VMG on the disc or 0 for file access           */
/* data will be filled with 2048 bytes of data on success                          */
/* return 0 on success, 1 on error                                                 */
/***********************************************************************************/
int DiscReadVMGM(long int lbnum, long int vmg_start, unsigned char *data);

/***********************************************************************************/
/* reads Logical Block (2048 byte) of a vts menu  (VTS_xx_0.VOB)                   */
/* vtsn is 1 thru 99 for VTS_xx_0.VOB (where xx is vtsn)                           */
/* lbnum is the logical block number in the VMG                                    */
/* vtsm_start is the position of the VTSM on the disc or 0 for file access         */
/* data will be filled with 2048 bytes of data on success                          */
/* return 0 on success, >0 on error                                                */
/***********************************************************************************/
int DiscReadVTSM(int vtsn, long int lbnum, long int vtsm_start, unsigned char *data);

/***********************************************************************************/
/* reads Logical Block (2048 byte) of a video title set  (VTS_xx_y.VOB)            */
/* vtsn is 1 thru 99 for VTS_xx_y.VOB (where xx is vtsn and y is 1 thru 9)         */
/* lbnum is the logical block number in the VMG                                    */
/* vts_start is the position of the VTS on the disc or 0 for file access           */
/* data will be filled with 2048 bytes of data on success                          */
/* return 0 on success, 1 on error                                                 */
/***********************************************************************************/
int DiscReadVTS(int vtsn, long int lbnum, long int vts_start, unsigned char *data);


/***********************************************************************************/
/* Access functions for DVD Audio are still to be defined. They will most likely   */
/* have the names DiscReadSAMG(), DiscReadAMG(), DiscReadAMGM() and DiscReadATS()  */
/***********************************************************************************/


/************************************/
/* DVD Copy Management:             */
/* RPC - Region Playback Control    */
/* CSS - Content Scrambling System  */
/************************************/

/***********************************************************************************/
/* Query RPC status of the drive                                                   */
/* type: 0=NONE (no drive region setting)                                          */
/*       1=SET (drive region is set                                                */
/*       2=LAST CHANCE (drive region is set, only one change remains)              */
/*       3=PERM (region set permanently, may be reset by vendor)                   */
/* vra: number of vendor resets available                                          */
/* ucca: number of user controlled changes available                               */
/* region_mask: the bit of the drive's region is set to 0, all other 7 bits to 1   */
/* rpc_scheme: 0=unknown, 1=RPC Phase II, others reserved                          */
/* returns 0 on success, <0 on error                                               */
/***********************************************************************************/
int DiscRPCGet(int *type, int *vra, int *ucca, int *region_mask, int *rpc_scheme);

/***********************************************************************************/
/* Set new Region for drive                                                        */
/* region_mask: the bit of the new drive's region is set to 0, all other 7 bits to 1 */
/***********************************************************************************/
int DiscRPCSet(int region_mask);

/***********************************************************************************/
/* Initializes the CSS process with the drive                                      */
/* returns positive AGID on success, -1 on error, -2 if no CSS support in CD-ROM driver */
/***********************************************************************************/
int DiscCSSRequestAGID(void);

/***********************************************************************************/
/* Invalidates previously released AGID                                            */
/* returns 0 on success, <0 on error                                               */
/***********************************************************************************/
int DiscCSSInvalidateAGID(int agid);

/***********************************************************************************/
/* Post host challenge (10 bytes) into drive                                       */
/* and retreive drive response (5 bytes) for host                                  */
/* returns 0 on success, -1 on error, -2 if no CSS support in CD-ROM driver        */
/***********************************************************************************/
int DiscCSSDriveAuth(char *data);

/***********************************************************************************/
/* Retreive drive challenge (10 bytes)                                             */
/* returns 0 on success, -1 on error, -2 if no CSS support in CD-ROM driver        */
/***********************************************************************************/
int DiscCSSHostAuthChallenge(char *data);

/***********************************************************************************/
/* Post host response (5 bytes) into drive                                         */
/* returns 0 on success, -1 on error, -2 if no CSS support in CD-ROM driver        */
/***********************************************************************************/
int DiscCSSHostAuthResponse(char *data);

/***********************************************************************************/
/* Retreive disc key (2048 byte) from drive                                        */
/* returns 0 on success, -1 on error, -2 if no CSS support in CD-ROM driver        */
/***********************************************************************************/
int DiscCSSDiscKey(char *data);

/***********************************************************************************/
/* Retreive title key (5 byte) from drive                                          */
/* input parameters:                                                               */
/*   lba: absolute number of logical block containing the title key                */
/* return parameters:                                                              */
/*   data: will be filled with the bus obfuscated title key                        */
/*   cpm: 0=material not copyrighted, 1=copyrighted material                       */
/*   cp_sec: 1=this sector has specific structure for copyright protection system, */
/*           0=it doesn't                                                          */
/*   cgms: 0=copying is permitted,                                                 */ 
/*         2=one generation of copies may be made,                                 */
/*         3=no copying is allowed                                                 */
/* returns 0 on success, <0 on error                                               */
/***********************************************************************************/
int DiscCSSTitleKey(unsigned long int lba, char *data, int *cpm, int *cp_sec, int *cgms);

/***********************************************************************************/
/* Set the path to the UDF mounted DVD disc                                        */
/* Or the Path to the VIDEO_TS directory                                           */
/* Or the filename of the udf block device or image file                           */
/* LB_length is the size of a Logical Block, default: 2048 byte                    */
/* returns 0 on success, -1 on 'dvd not found', -2 on 'path not found'             */
/***********************************************************************************/
int DiscSetVideoPath(char *Path, int LB_Length);

#endif  /* DVD_DISC_H */
