#ifndef DVD_FILE_H
#define DVD_FILE_H

#define DVD_FILE_VERSION 20000215


/***********************************************************************************/
/* Reads a whole file, e.g. an IFO File or IFO Backup File                         */
/* filename_ifo is the name of the file to read                                    */
/* *infodata will be allocated to the size of the file                             */
/* returns 0 on error, number of bytes read on success                             */
/***********************************************************************************/
int FileReadFile(char *filename, unsigned char **infodata);

/***********************************************************************************/
/* reads Logical Block (2048 byte) of a vmg menu (VIDEO_TS.VOB)                    */
/* lbnum is the logical block number in the VMG                                    */
/* data will be filled with 2048 bytes of data on success                          */
/* return 0 on success, 1 on error                                                 */
/***********************************************************************************/
int FileReadVMGM(long int lbnum, unsigned char *data);

/***********************************************************************************/
/* reads Logical Block (2048 byte) of a vts menu                                   */
/* vtsn is 1 thru 99 for VTS_xx_0.VOB (where xx is vtsn)                           */
/* lbnum is the logical block number in the VTSM                                   */
/* data will be filled with 2048 bytes of data on success                          */
/* return 0 on success, >0 on error                                                */
/***********************************************************************************/
int FileReadVTSM(int vtsn, long int lbnum, unsigned char *data);

/***********************************************************************************/
/* reads Logical Block (2048 byte) of a video title set                            */
/* vtsn is 1 thru 99 for VTS_xx_y.VOB (where xx is vtsn and y is 1 thru 9)         */
/* lbnum is the logical block number in the VTS                                    */
/* data will be filled with 2048 bytes of data on success                          */
/* return 0 on success, 1 on error                                                 */
/***********************************************************************************/
int FileReadVTS(int vtsn, long int lbnum, unsigned char *data);

/***********************************************************************************/
/* Set the path to the UDF mounted DVD disc                                        */
/* Or the Path to the VIDEO_TS directory                                           */
/* LB_length is the size of a Logical Block, default: 2048 byte                    */
/* returns 0 on success, -1 on 'dvd not found', -2 on 'path not found'             */
/***********************************************************************************/
int FileSetVideoPath(char *Path, int LB_Length);

#endif  /* DVD_FILE_H */
