#ifndef DVD_UDF_H
#define DVD_UDF_H

#define DVD_UDF_VERSION 20000215


/***********************************************************************************/
/* The length of one Logical Block of a DVD Video                                  */
/***********************************************************************************/
#define DVD_VIDEO_LB_LEN 2048

/***********************************************************************************/
/* reads Logical Block of the disc or image                                        */
/*   lb_number: disc-absolute logical block number                                 */
/*   block_count: number of 2048 byte blocks to read                               */
/*   data: pointer to enough allocated memory                                      */
/*   returns number of read bytes on success, 0 or negative error number on error  */
/***********************************************************************************/
int UDFReadLB(unsigned long int lb_number, unsigned int block_count, unsigned char *data);

/***********************************************************************************/
/* looks for a file on the UDF disc/imagefile                                      */
/*   filename: absolute pathname on the UDF filesystem, starting with '/'          */
/*   filesize will be set to the size of the file in bytes, on success             */
/*   returns absolute LB number, or 0 on error                                     */
/***********************************************************************************/
unsigned long int UDFFindFile(char *filename, unsigned long int *filesize);


/************************************/
/* DVD Copy Management:             */
/* RPC - Region Playback Control    */
/* CSS - Content Scrambling System  */
/************************************/

/***********************************************************************************/
/* Query RPC status of the drive                                                   */
/* type: 0=NONE (no drive region setting)                                          */
/*       1=SET (drive region is set                                                */
/*       2=LAST CHANCE (drive region is set, only one change remains)              */
/*       3=PERM (region set permanently, may be reset by vendor)                   */
/* vra: number of vendor resets available                                          */
/* ucca: number of user controlled changes available                               */
/* region_mask: the bit of the drive's region is set to 0, all other 7 bits to 1   */
/* rpc_scheme: 0=unknown, 1=RPC Phase II, others reserved                          */
/* returns 0 on success, <0 on error                                               */
/***********************************************************************************/
int UDFRPCGet(int *type, int *vra, int *ucca, int *region_mask, int *rpc_scheme);

/***********************************************************************************/
/* Set new Region for drive                                                        */
/* region_mask: the bit of the new drive's region is set to 0, all other 7 bits to 1 */
/***********************************************************************************/
int UDFRPCSet(int region_mask);

/***********************************************************************************/
/* Initializes the CSS process with the drive                                      */
/* returns positive AGID on success, -1 on error, -2 if no CSS support in CD-ROM driver */
/***********************************************************************************/
int UDFCSSRequestAGID(void);

/***********************************************************************************/
/* Invalidates previously released AGID                                            */
/* returns 0 on success, <0 on error                                               */
/***********************************************************************************/
int UDFCSSInvalidateAGID(int agid);

/***********************************************************************************/
/* Post host challenge (10 bytes) into drive                                       */
/* and retreive drive response (5 bytes) for host                                  */
/* returns 0 on success, -1 on error, -2 if no CSS support in CD-ROM driver        */
/***********************************************************************************/
int UDFCSSDriveAuth(char *data);

/***********************************************************************************/
/* Retreive drive challenge (10 bytes)                                             */
/* returns 0 on success, -1 on error, -2 if no CSS support in CD-ROM driver        */
/***********************************************************************************/
int UDFCSSHostAuthChallenge(char *data);

/***********************************************************************************/
/* Post host response (5 bytes) into drive                                         */
/* returns 0 on success, -1 on error, -2 if no CSS support in CD-ROM driver        */
/***********************************************************************************/
int UDFCSSHostAuthResponse(char *data);

/***********************************************************************************/
/* Retreive disc key (2048 byte) from drive                                        */
/* returns 0 on success, -1 on error, -2 if no CSS support in CD-ROM driver        */
/***********************************************************************************/
int UDFCSSDiscKey(char *data);

/***********************************************************************************/
/* Retreive title key (5 byte) from drive                                          */
/* input parameters:                                                               */
/*   lba: absolute number of logical block containing the title key                */
/* return parameters:                                                              */
/*   data: will be filled with the bus obfuscated title key                        */
/*   cpm: 0=material not copyrighted, 1=copyrighted material                       */
/*   cp_sec: 1=this sector has specific structure for copyright protection system, */
/*           0=it doesn't                                                          */
/*   cgms: 0=copying is permitted,                                                 */ 
/*         2=one generation of copies may be made,                                 */
/*         3=no copying is allowed                                                 */
/* returns 0 on success, <0 on error                                               */
/***********************************************************************************/
int UDFCSSTitleKey(unsigned long int lba, char *data, int *cpm, int *cp_sec, int *cgms);


/***********************************************************************************/
/* opens block device or image file                                                */
/*   filename: path to the DVD ROM block device or to the image file in UDF format */
/*   returns fileno() of the file on success, or -1 on error                       */
/***********************************************************************************/
int UDFOpenDisc(char *filename);

/***********************************************************************************/
/* closes previously opened block device or image file                             */
/*   returns 0 on success, or -1 on error                                          */
/***********************************************************************************/
int UDFCloseDisc(void);

#endif /* DVD_UDF_H */
