/*
 * dvdbackup: DVD backup maker
 * Copyright (C) 1999 Christian Wolff for convergence integrated media GmbH
 * 1999-12-20
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
 * 
 * The author can be reached at scarabaeus@convergence.de, 
 * the project's page is at http://linuxtv.org/dvd/
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <linux/limits.h>

#include "dvd_udf.h"

// If UDF_SOFT_CSS is defined, CSS can be done in software, 
// The unencrypted keys will be stored in an extra file
//#define UDF_SOFT_CSS

#ifdef UDF_SOFT_CSS
#include <dvd_css.h>
#endif

#define DEFAULTNAME   "DVDVIDEO"
#define DEFAULTSIZE   650*1024*1024

#ifdef UDF_SOFT_CSS

int decrypt=1;  // decrypt by default

// CSS key exchange
// disc: 1=disc key, 2=title key
// lba: logical block address of title (usually any lba of VTS_xx_1.VOB thru VTS_xx_9.VOB)
int backupCSS(int disc, u32 lba) 
{
	char CSSdata[2048];
	int agid,cpm,cp_sec,cgms;
	if ((agid=UDFCSSRequestAGID())<0) {
		return -1;
	}
	if (CSSDriveAuthChallenge(CSSdata)) goto ERROR;
	if (UDFCSSDriveAuth(CSSdata)<0) goto ERROR;
	if (CSSDriveAuthResponse(CSSdata)) goto ERROR;
	if (UDFCSSHostAuthChallenge(CSSdata)<0) goto ERROR;
	if (CSSHostAuth(CSSdata)) goto ERROR;
	if (UDFCSSHostAuthResponse(CSSdata)<0) goto ERROR;
	if (disc==1) {  // do disc authentication
		if (UDFCSSDiscKey(CSSdata)<0) goto ERROR;
		if (CSSHostDiscKey(CSSdata)) goto ERROR;
	} else if (disc==2) {  // title authentication
		if (UDFCSSTitleKey(lba,CSSdata,&cpm,&cp_sec,&cgms)<0) goto ERROR;
		//printf("CSS - Copyright info: cpm:%d, cp_sec:%d, cgms:%d\n",cpm,cp_sec,cgms);
		if (CSSHostTitleKey(lba,CSSdata)) goto ERROR;
	}
	UDFCSSInvalidateAGID(agid);
	return 0;
	ERROR:
	UDFCSSInvalidateAGID(agid);
	return -1;
}

// authorizes DVD disc and write keys to filename.css
int authorize_css(char *filename) 
{
	char file[PATH_MAX];
	u32 lba,filesize;
	int i,err=0;
	CSSHostReset();
	// Disc Key
	printf("Reading disc key\n");
	if (backupCSS(1,0)<0) {  // retreive disc key
		printf("Disc key not found, disc is not CSS encrypted.\n");
		return 1;
	}
	// retreive all title keys
	// Title Key of Video Manager
	strcpy(file,"/VIDEO_TS/VIDEO_TS.VOB");
	if ((lba=UDFFindFile(file,&filesize))>0) {
		printf("  Reading title key for %s at lba %ld\n",file,lba);
		backupCSS(2,lba);  // retreive title key
	}
	// look for all title sets from 1 to 99, stop if neither VTS nor VTSM are found
	for (i=1; (i<100) && (err<2); i++) {
		err=0;
		// Title Key of Video Title Set Manager
		sprintf(file,"/VIDEO_TS/VTS_%02d_0.VOB",i);
		if ((lba=UDFFindFile(file,&filesize))>0) {
			printf("  Reading title key for %s at lba %ld\n",file,lba);
			backupCSS(2,lba);  // retreive title key
		} else err++;
		// Title Key of Video Title Set
		sprintf(file,"/VIDEO_TS/VTS_%02d_1.VOB",i);
		if ((lba=UDFFindFile(file,&filesize))>0) {
			printf("  Reading title key for %s at lba %ld\n",file,lba);
			backupCSS(2,lba);  // retreive title key
		} else err++;
	}
	CSSHostWriteKeys(filename);
	return 0;
}

#endif


// filename: path and name. '.000', '.001' etc will be appended
// segmentsize: maximum size in bytes of one segment
// returns 0 on OK, 1 on error
int splitcopy(char *filename, unsigned int segmentsize) 
{
	char file[PATH_MAX];
	FILE *segfile;
	int i,copied,segment=0,blockcount,err=0,buffersize;
	unsigned char buffer[DVD_VIDEO_LB_LEN*256];
	unsigned long int lb_number=0;
	char factor[2];
	char mark[2];
	char *wobble="_,-^-,";
	int wobpos,wobsize;
	
#ifdef UDF_SOFT_CSS
	authorize_css(filename);
#endif
	buffersize=sizeof(buffer)/DVD_VIDEO_LB_LEN;  // logical blocks in buffer
	segmentsize/=DVD_VIDEO_LB_LEN; 
	strcpy(factor,"");
	i=segmentsize*DVD_VIDEO_LB_LEN;
	if (i>1024) {
		i/=1024;
		strcpy(factor,"K");
	}
	if (i>1024) {
		i/=1024;
		strcpy(factor,"M");
	}
	if (i>1024) {
		i/=1024;
		strcpy(factor,"G");
	}
	printf("Writing DVD, segmentsize is %d %sByte, each mark is %d bytes\n",i,factor,buffersize*DVD_VIDEO_LB_LEN);
	wobsize=strlen(wobble);
	wobpos=0;
	while (!err) {
		sprintf(file,"%s.%03d",filename,segment++);
		if ((segfile=fopen(file,"w"))==NULL) {
			printf("File %s could not be created!\n",file);
			return 1;
		} else {
			printf("Writing segment #%d: %s\n",segment,file);
		}
		copied=0;
		while ((!err) && (copied<segmentsize)) {
			strncpy(mark,&wobble[wobpos++],1);
			wobpos%=wobsize;
			blockcount=((copied+buffersize>segmentsize)?segmentsize-copied:buffersize);
			if (UDFReadLB(lb_number,blockcount,buffer)>0) {
#ifdef UDF_SOFT_CSS
				if (decrypt) {
					if (CSSHostDecrypt(lb_number,blockcount,buffer)) strcpy(mark,"c");
				}
#endif
				lb_number+=blockcount;
				if (fwrite(buffer,blockcount*DVD_VIDEO_LB_LEN,1,segfile)>0) {
					copied+=blockcount;
					printf("%s",mark);
				} else {
					printf("\nFile %s could not be written!\n",file);
					return 1;
				}
			} else {
				err=1;
			}
		}
		printf("\n");
	}
	printf("Copied %ld blocks total into %d segment files\n",lb_number,segment);
	return 0;
}

int main (int argc, char* argv[]) 
{
	int err,segsize;
	setbuf(stdout,NULL);
	printf("DVD Backup maker\n");
	printf("  usage: dvdbackup <dvd device> [<target file> [<segment size>[K|M|G]]]\n"); 
	printf("  Default target file:  %s\n",DEFAULTNAME);
	printf("  Default segment size: %d MByte\n",DEFAULTSIZE/1024/1024);
	if (argc<=1) return -1; 
	err=UDFOpenDisc(argv[1]);
	if (err<0) {
		printf("ERROR: Could not open disc \"%s\"!\n",argv[1]);
		return 1;
	}
	if (argc>2) {
		if (argc>3) {
#ifdef UDF_SOFT_CSS
			// passing a fourth argument disables CSS decryption.
			// an encrypted image will be written instead.
			if (argc>4) decrypt=0;
#endif
			if ((argv[3][strlen(argv[3])-1]=='k') || (argv[3][strlen(argv[3])-1]=='K')) {
				argv[3][strlen(argv[3])-1]='\0';
				segsize=1024*atoi(argv[3]);
			} else if ((argv[3][strlen(argv[3])-1]=='m') || (argv[3][strlen(argv[3])-1]=='M')) {
				argv[3][strlen(argv[3])-1]='\0';
				segsize=1024*1024*atoi(argv[3]);
			} else if ((argv[3][strlen(argv[3])-1]=='g') || (argv[3][strlen(argv[3])-1]=='G')) {
				argv[3][strlen(argv[3])-1]='\0';
				segsize=1024*1024*1024*atoi(argv[3]);
			} else {
				segsize=atoi(argv[3]);
			}
			splitcopy(argv[2],segsize);
		} else {
			splitcopy(argv[2],DEFAULTSIZE);
		}
	} else {
		splitcopy(DEFAULTNAME,DEFAULTSIZE);
	}
	UDFCloseDisc();
	return 0;
}
